@@ -89,7 +89,14 @@
 /*	with the command specified with $maillog_file_compressor.
 /*	This will not rotate /dev/* files.
 /* .sp
-/*	This feature is available in Postfix 3.4 and later.
+/* .IP "\fBnon-bdb\fR \fIsubcommand\fR"
+/*	Migrate a Postfix configuration that uses Berkeley DB hash:
+/*	or btree: tables, to a configuration that uses lmdb: or a
+/*	combination of cdb: and lmdb:. This is needed because some
+/*	(Linux) distributions have removed Berkeley DB support.
+/*	See postfix-nbdb(1) for documentation.
+/* .sp
+/*	This feature is available in Postfix 3.11 and later.
 /* .IP "\fBtls\fR \fIsubcommand\fR"
 /*	Enable opportunistic TLS in the Postfix SMTP client or
 /*	server, and manage Postfix SMTP server TLS private keys and
@@ -303,7 +310,7 @@
 /*
 /*	Table-driven mechanisms:
 /*	access(5), Postfix SMTP access control table
-/*	aliases(5), Postfix alias database
+/*	aliases(5), Postfix local aliasing
 /*	canonical(5), Postfix input address rewriting
 /*	generic(5), Postfix output address rewriting
 /*	header_checks(5), body_checks(5), Postfix content inspection
@@ -316,6 +323,7 @@
 /*	ldap_table(5), Postfix LDAP client
 /*	lmdb_table(5), Postfix LMDB database driver
 /*	memcache_table(5), Postfix memcache client
+/*	mongodb_table(5), Postfix MongoDB client
 /*	mysql_table(5), Postfix MYSQL client
 /*	nisplus_table(5), Postfix NIS+ client
 /*	pcre_table(5), Associate PCRE pattern with value
@@ -412,8 +420,6 @@
 /*	Jun-ichiro 'itojun' Hagino, KAME project, Japan
 /*	The Linux PLD project
 /*	Dean Strik, Eindhoven University, The Netherlands
-/*
-/*	Wietse Venema
 /*--*/
 
 /* System library. */
@@ -591,7 +597,7 @@ int     main(int argc, char **argv)
      */
     if (compat_level < compat_level_from_string(LAST_COMPAT_LEVEL, msg_panic)) {
 	msg_info("Postfix is using backwards-compatible default settings");
-	msg_info("See http://www.postfix.org/COMPATIBILITY_README.html "
+	msg_info("See https://www.postfix.org/COMPATIBILITY_README.html "
 		 "for details");
 	msg_info("To disable backwards compatibility use \"postconf "
 		 VAR_COMPAT_LEVEL "=%s\" and \"postfix reload\"",
@@ -636,8 +642,7 @@ int     main(int argc, char **argv)
      * Run the management script.
      */
     if (force_single_instance
-	|| argv_split_cw(var_multi_conf_dirs, CHARS_COMMA_SP,
-			 VAR_MULTI_CONF_DIRS)->argc == 0) {
+	|| argv_split(var_multi_conf_dirs, CHARS_COMMA_SP)->argc == 0) {
 	script = concatenate(var_daemon_dir, "/postfix-script", (char *) 0);
 	if (optind < 1)
 	    msg_panic("bad optind value");
